/**
 * Copyright (c) 2023, WSO2 LLC. (https://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 LLC. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content.
 */

import { IdentityAppsApiException } from "@wso2is/core/exceptions";
import { hasRequiredScopes } from "@wso2is/core/helpers";
import { AlertInterface, AlertLevels, IdentifiableComponentInterface } from "@wso2is/core/models";
import { addAlert } from "@wso2is/core/store";
import {
    ConfirmationModal,
    ContentLoader,
    EmphasizedSegment,
    Heading
} from "@wso2is/react-components";
import { getOIDCData } from "apps/console/src/features/applications/api";
import { OIDCDataInterface } from "apps/console/src/features/applications/models";
import { RequestErrorInterface } from "apps/console/src/features/core/hooks/use-request";
import { AxiosError } from "axios";
import React, {
    Fragment,
    FunctionComponent,
    ReactElement,
    useEffect,
    useState
} from "react";
import { useTranslation } from "react-i18next";
import { useDispatch, useSelector } from "react-redux";
import { Dispatch } from "redux";
import { Divider, Grid } from "semantic-ui-react";
import { PolicySection, SubscribedAPIResources } from ".";
import { AuthorizeAPIResource } from "./wizard";
import { AppState, history } from "../../../../../features/core";
import { ExtendedFeatureConfigInterface } from "../../../../configs/models";
import { useAPIResources } from "../../../api-resources/api";
import { APIResourcesConstants } from "../../../api-resources/constants";
import { 
    createAuthorizedAPIResource, 
    patchPolicies, 
    unsubscribeAPIResources, 
    usePolicies, 
    useSubscribedAPIResources 
} from "../../api";
import { APIAuthorizationConstants } from "../../constants";
import { AuthorizedAPIListItemInterface, AuthorizedPermissionListItemInterface } from "../../models";

/**
 * Application roles component.
 * 
 * @param props - Props related to application roles component.
 */
export const APIAuthorization: FunctionComponent<IdentifiableComponentInterface> = (
    props: IdentifiableComponentInterface
): ReactElement => {

    const {
        ["data-componentid"]: componentId
    } = props;

    const { t } = useTranslation();
    const dispatch: Dispatch = useDispatch();

    const [ isSubAPIResourcesSectionLoading, setSubAPIResourcesSectionLoading ] = useState<boolean>(false);
    const [ isPolicySectionLoading, setPolicySectionLoading ] = useState<boolean>(false);
    const [ isShownError, setIsShownError ] = useState<boolean>(false);
    const [ oidcData, setOidcData ] = useState<OIDCDataInterface>(undefined);
    const [ isClientCreditionalsGrantEnabled, setIsClientCreditionalsGrantEnabled ] = useState<boolean>(false);
    const [ removeSubscribedAPIResource, setRemoveSubscribedAPIResource ] = 
        useState<AuthorizedAPIListItemInterface>(null);
    const [ isUnsubscribeAPIResourceLoading, setIsUnsubscribeAPIResourceLoading ] = useState<boolean>(false);
    const [ isAuthorizeAPIResourceWizardOpen, setIsAuthorizeAPIResourceWizardOpen ] = useState<boolean>(false);
    const [ isUpdateData, setIsUpdateData ] = useState<boolean>(false);
    const [ allAuthorizedScopes, setAllAuthorizedScopes ] = useState<AuthorizedPermissionListItemInterface[]>([]);

    const featureConfig: ExtendedFeatureConfigInterface = useSelector((state: AppState) => state.config.ui.features);
    const allowedScopes: string = useSelector((state: AppState) => state?.auth?.allowedScopes);

    const path: string[] = history.location.pathname.split("/");
    const appId: string = path[path.length - 1].split("#")[0];

    const {
        data: allAPIResourcesListData,
        isLoading: isAllAPIResourcesListLoading,
        error: allAPIResourcesFetchRequestError,
        mutate: mutateAllAPIResourcesList
    } = useAPIResources();

    const {
        data: subscribedAPIResourcesListData,
        isLoading: isSubscribedAPIResourcesListLoading,
        error: subscribedAPIResourcesFetchRequestError,
        mutate: mutateSubscribedAPIResourcesList
    } = useSubscribedAPIResources(appId);

    const {
        data: policies,
        isLoading: isPoliciesLoading,
        error: policiesError,
        mutate: mutatePolicies
    } = usePolicies(appId);

    /**
     * The following method is used to get OIDC data of the application.
     * 
     * @returns `void`
     */
    const getOIDCDataOfTheApplication = (): void => {
        getOIDCData(appId)
            .then((response: OIDCDataInterface) => {
                setOidcData(response);
            })
            .catch(() => {
                setOidcData(undefined);
            });
    };

    /**
     * Handles the hide of the policy selection./
     */
    const isHidePolicySelection: boolean = !!subscribedAPIResourcesFetchRequestError
        || !!allAPIResourcesFetchRequestError
        || allAPIResourcesListData?.apiResources.length === 0 
        || subscribedAPIResourcesListData?.length === 0;

    /**
     * The following useEffect is used to get OIDC data of the application.
     */
    useEffect(() => {
        getOIDCDataOfTheApplication();
    }, []);

    /**
     * The following useEffect is used to check if the client credentials grant is enabled.
     */
    useEffect(() => {
        if (oidcData) {
            const grantTypes: string[] = oidcData.grantTypes;

            if (grantTypes.includes(APIAuthorizationConstants.CLIENT_CREDENTIALS_GRANT_TYPE)) {
                setIsClientCreditionalsGrantEnabled(true);
            } else {
                setIsClientCreditionalsGrantEnabled(false);
            }
        }
    }, [ oidcData ]);

    /**
     * The following useEffect is used to handle if any error occurs while fetching API resources.
     */
    useEffect(() => {
        if (allAPIResourcesFetchRequestError || subscribedAPIResourcesFetchRequestError) {
            if (!isShownError) {
                setIsShownError(true);

                const error: AxiosError<RequestErrorInterface> = 
                    allAPIResourcesFetchRequestError 
                        ? allAPIResourcesFetchRequestError 
                        : subscribedAPIResourcesFetchRequestError;

                switch (error.response?.data?.code) {
                    case APIResourcesConstants.UNAUTHORIZED_ACCESS:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.apiResource.notifications.getAPIResources" +
                                ".unauthorizedError.description"),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.apiResource.notifications.getAPIResources" +
                                ".unauthorizedError.message")
                        }));

                        break;

                    default:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.apiResource.notifications.getAPIResources" +
                                ".genericError.description"),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.apiResource.notifications.getAPIResources" +
                                ".genericError.message")
                        }));
                }
            }
        }
    }, [ allAPIResourcesFetchRequestError, subscribedAPIResourcesFetchRequestError ]);

    /**
     * The following useEffect is used to handle if any error occurs while fetching the policies.
     */
    useEffect(() => {
        if (policiesError) {
            dispatch(addAlert<AlertInterface>({
                description: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                    ".sections.policySection.notifications.getPolicies.genericError.description"),
                level: AlertLevels.ERROR,
                message: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                ".sections.policySection.notifications.getPolicies.genericError.message")
            }));
        }
    }, [ policiesError ]);

    /**
     * The following useEffect is used to handle loading state of when API resources are being fetched.
     */
    useEffect(() => {
        setSubAPIResourcesSectionLoading(isUpdateData || 
            isAllAPIResourcesListLoading || isSubscribedAPIResourcesListLoading);
    }, [ isUpdateData, isAllAPIResourcesListLoading, isSubscribedAPIResourcesListLoading ]);

    /**
     * Initalize the all authorized scopes.
     */
    useEffect(() => {
        if (subscribedAPIResourcesListData?.length > 0) {
            let authorizedScopes: AuthorizedPermissionListItemInterface[] = [];
            
            subscribedAPIResourcesListData.forEach((subscribedAPIResource: AuthorizedAPIListItemInterface) => {
                authorizedScopes = authorizedScopes.concat(subscribedAPIResource.permissions);
            });
            
            setAllAuthorizedScopes(authorizedScopes);
        }
    }, [ subscribedAPIResourcesListData ]);

    /**
     * The following useEffect is used to update the API resources list once a mutate function is called.
     */
    useEffect(() => {
        if (isUpdateData) {
            mutateAllAPIResourcesList();
            mutateSubscribedAPIResourcesList();
            setIsUpdateData(false);
        }
    }, [ isUpdateData ]);

    /**
     * Handles unsubscribe API resource.
     *
     * @returns `void`
     */
    const handleAPIResourceUnsubscribe = (): void => {

        setIsUnsubscribeAPIResourceLoading(true);
        unsubscribeAPIResources(appId, removeSubscribedAPIResource?.apiId)
            .then(() => {
                dispatch(addAlert<AlertInterface>({
                    description: t("extensions:develop.applications.edit.sections." +
                        "apiAuthorization.sections.apiSubscriptions.notifications.unSubscribe.success.description"),
                    level: AlertLevels.SUCCESS,
                    message: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                        ".apiSubscriptions.notifications.unSubscribe.success.message")
                }));

                setIsUnsubscribeAPIResourceLoading(false);
                setIsUpdateData(true);
            })
            .catch((error: IdentityAppsApiException) => {
                switch(error?.code) {
                    case APIResourcesConstants.UNAUTHORIZED_ACCESS:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.notifications.unSubscribe.unauthorizedError.description"),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.notifications.unSubscribe.unauthorizedError.message")
                        }));

                        break;
                        
                    case APIResourcesConstants.NO_VALID_API_RESOURCE_ID_FOUND:
                    case APIResourcesConstants.API_RESOURCE_NOT_FOUND:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.notifications.unSubscribe.notFoundError.description"),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.notifications.unSubscribe.notFoundError.message")
                        }));

                        break;

                    default:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.notifications.unSubscribe.genericError.description"),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.notifications.unSubscribe.genericError.message")
                        }));
                }
            })
            .finally(() => {
                setIsUnsubscribeAPIResourceLoading(false);
                setRemoveSubscribedAPIResource(null);
            });
    };

    /**
     * Handles creation of authorized API resource.
     *
     * @param apiId - API resource ID.
     * @param scopes - Scopes.
     * @returns `void`
     */
    const handleCreateAPIResource = (apiId: string, scopes: string[], callback: () => void): void => {

        createAuthorizedAPIResource(appId, apiId, scopes)
            .then(() => {
                dispatch(addAlert<AlertInterface>({
                    description: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                        ".sections.apiSubscriptions.notifications.createAuthorizedAPIResource.success.description"),
                    level: AlertLevels.SUCCESS,
                    message: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                        ".apiSubscriptions.notifications.createAuthorizedAPIResource.success.message")
                }));

                setIsUpdateData(true);
                setIsAuthorizeAPIResourceWizardOpen(false);
            })
            .catch((error: IdentityAppsApiException) => {
                switch(error?.code) {
                    case APIResourcesConstants.UNAUTHORIZED_ACCESS:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.notifications.createAuthorizedAPIResource.unauthorizedError" + 
                                ".description"),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.notifications.createAuthorizedAPIResource.unauthorizedError.message")
                        }));

                        break;
                        
                    case APIResourcesConstants.NO_VALID_API_RESOURCE_ID_FOUND:
                    case APIResourcesConstants.API_RESOURCE_NOT_FOUND:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.notifications.createAuthorizedAPIResource.notFoundError" + 
                                ".description"),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.notifications.createAuthorizedAPIResource.notFoundError.message")
                        }));

                        break;

                    default:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.notifications.createAuthorizedAPIResource.genericError.description"),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.notifications.createAuthorizedAPIResource.genericError.message")
                        }));
                }
            })
            .finally(() => {
                setIsUnsubscribeAPIResourceLoading(false);
                callback && callback();
            });
    };

    /**
     * Check scopes available for update API resources.
     * 
     * @returns `boolean` `true` if scopes available for update.
     */
    const isScopesAvailableForUpdate = (): boolean => {
        return hasRequiredScopes(featureConfig?.applications,
            featureConfig?.applications?.scopes?.update, allowedScopes);
    };

    /**
     * Update all authorized scopes.
     * 
     * @param updatedScope - Updated scope.
     * @param removed - `true` if scope removed.
     * 
     * @returns `void`
     */
    const updateAllAuthorizedScopes = (updatedScope: AuthorizedPermissionListItemInterface, removed: boolean): void => {
        if (removed) {
            setAllAuthorizedScopes(allAuthorizedScopes.filter(
                (scope: AuthorizedPermissionListItemInterface) => scope.name !== updatedScope.name)
            );
        } else {
            setAllAuthorizedScopes([ ...allAuthorizedScopes, updatedScope ]);
        }
    };

    /**
     * Handles updating the policies of the application.
     *
     * @param addedPolicies - Added policies.
     * @param removedPolicies - Removed policies.
     * 
     * @returns `void`
     */
    const handleUpdatePolicies = (addedPolicies: string[], removedPolicies: string[]): void => {

        patchPolicies(appId, addedPolicies, removedPolicies)
            .then(() => {
                dispatch(addAlert<AlertInterface>({
                    description: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                        ".sections.policySection.notifications.patchPolicies.success.description"),
                    level: AlertLevels.SUCCESS,
                    message: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                        ".sections.policySection.notifications.patchPolicies.success.message")
                }));

                mutatePolicies();
            })
            .catch((error: IdentityAppsApiException) => {
                switch(error?.code) {
                    case APIResourcesConstants.UNAUTHORIZED_ACCESS:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                ".sections.policySection.notifications.patchPolicies.unauthorizedError.description"),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                ".sections.policySection.notifications.patchPolicies.unauthorizedError.message")
                        }));

                        break;

                    default:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                ".sections.policySection.notifications.patchPolicies.genericError.description"),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                ".sections.policySection.notifications.patchPolicies.genericError.message")
                        }));
                }
            })
            .finally(() => {
                setPolicySectionLoading(false);
            });
    };

    /**
     * Bulk change the all authorized scopes.
     * 
     * @param updatedScopes - Updated scopes.
     * @param removed - `true` if scope removed.
     * 
     * @returns `void`
     */
    const bulkChangeAllAuthorizedScopes = (updatedScopes: AuthorizedPermissionListItemInterface[], 
        removed: boolean): void => {

        if (removed) {
            setAllAuthorizedScopes(allAuthorizedScopes.filter(
                (scope: AuthorizedPermissionListItemInterface) => !updatedScopes.some(
                    (updatedScope: AuthorizedPermissionListItemInterface) => updatedScope.name === scope.name)));
        } else {
            const changedScopes: AuthorizedPermissionListItemInterface[]= updatedScopes.filter(
                (updatedScope: AuthorizedPermissionListItemInterface) => !allAuthorizedScopes.some(
                    (scope: AuthorizedPermissionListItemInterface) => updatedScope.name === scope.name));

            setAllAuthorizedScopes([ ...allAuthorizedScopes, ...changedScopes ]);
        }
    };

    return (
        <Fragment>
            <EmphasizedSegment padded="very" data-componentid={ `${componentId}-sub-api-resources-section` }>
                <Grid>
                    <Grid.Row>
                        <Grid.Column className="heading-wrapper" computer={ 10 }>
                            <Heading as="h4">
                                { t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                    ".apiSubscriptions.heading") }
                            </Heading>
                            <Heading subHeading ellipsis as="h6" >
                                { t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                    ".apiSubscriptions.subHeading") }
                            </Heading>
                        </Grid.Column >
                    </Grid.Row>
                </Grid>
                <Divider hidden />
                <Grid>
                    <Grid.Row>
                        <Grid.Column>
                            {
                                isSubAPIResourcesSectionLoading
                                    ? <ContentLoader inline="centered" active />
                                    : (<SubscribedAPIResources
                                        appId= { appId }
                                        allAPIResourcesListData={ allAPIResourcesListData?.apiResources }
                                        allAPIResourcesFetchRequestError={ allAPIResourcesFetchRequestError }
                                        allAuthorizedScopes={ allAuthorizedScopes }
                                        subscribedAPIResourcesListData={ subscribedAPIResourcesListData }
                                        subscribedAPIResourcesFetchRequestError=
                                            { subscribedAPIResourcesFetchRequestError }
                                        isScopesAvailableForUpdate={ isScopesAvailableForUpdate() }
                                        isShownPlaceholder={ isHidePolicySelection }
                                        setRemoveSubscribedAPIResource={ setRemoveSubscribedAPIResource }
                                        updateAllAuthorizedScopes={ updateAllAuthorizedScopes }
                                        bulkChangeAllAuthorizedScopes={ bulkChangeAllAuthorizedScopes }
                                        setIsAuthorizeAPIResourceWizardOpen={ setIsAuthorizeAPIResourceWizardOpen }
                                    />)
                            }
                        </Grid.Column>
                    </Grid.Row>
                </Grid>
            </EmphasizedSegment>
            {
                (!isHidePolicySelection && !policiesError && !isSubAPIResourcesSectionLoading) && (
                    <EmphasizedSegment padded="very" data-componentid={ `${componentId}-policy-section` }>
                        <Grid>
                            <Grid.Row>
                                <Grid.Column className="heading-wrapper" computer={ 10 }>
                                    <Heading as="h4">
                                        { t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                            ".policySection.heading") }
                                    </Heading>
                                    <Heading subHeading ellipsis as="h6" >
                                        { t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                            ".policySection.subHeading") }
                                    </Heading>
                                </Grid.Column >
                            </Grid.Row>
                        </Grid>
                        <Divider hidden />
                        <Grid>
                            <Grid.Row>
                                <Grid.Column>
                                    <PolicySection
                                        policies={ policies?.policies }
                                        isPoliciesLoading={ isPoliciesLoading || isPolicySectionLoading }
                                        isClientCredentialsEnabled={ isClientCreditionalsGrantEnabled }
                                        isScopesAvailableForUpdate={ isScopesAvailableForUpdate() }
                                        setPolicySectionLoading={ setPolicySectionLoading }
                                        handleUpdatePolicies={ handleUpdatePolicies } />
                                </Grid.Column>
                            </Grid.Row>
                        </Grid>
                    </EmphasizedSegment>
                )
            }

            {
                removeSubscribedAPIResource && (
                    <ConfirmationModal
                        primaryActionLoading={ isUnsubscribeAPIResourceLoading }
                        open={ removeSubscribedAPIResource !== null }
                        onClose={ (): void => setRemoveSubscribedAPIResource(null) }
                        type="negative"
                        assertionHint={ t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                            ".apiSubscriptions.confirmations.unsubscribeAPIResource.assertionHint") }
                        assertionType="checkbox"
                        primaryAction={ t("common:confirm") }
                        secondaryAction={ t("common:cancel") }
                        onSecondaryActionClick={ (): void => setRemoveSubscribedAPIResource(null) }
                        onPrimaryActionClick={ (): void => handleAPIResourceUnsubscribe() }
                        data-componentid={ `${componentId}-delete-confirmation-modal` }
                        closeOnDimmerClick={ false }
                    >
                        <ConfirmationModal.Header
                            data-componentid={ `${componentId}-delete-confirmation-modal-header` }
                        >
                            { t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.confirmations.unsubscribeAPIResource.header") }
                        </ConfirmationModal.Header>
                        <ConfirmationModal.Message
                            attached
                            negative
                            data-componentid={ `${componentId}-delete-confirmation-modal-message` }
                        >
                            { t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.confirmations.unsubscribeAPIResource.message") }
                        </ConfirmationModal.Message>
                        <ConfirmationModal.Content
                            data-componentid={ `${componentId}-delete-confirmation-modal-content` }
                        >
                            { t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                                ".apiSubscriptions.confirmations.unsubscribeAPIResource.content") }
                        </ConfirmationModal.Content>
                    </ConfirmationModal>
                )
            }
            {
                isAuthorizeAPIResourceWizardOpen && (
                    <AuthorizeAPIResource 
                        subscribedAPIResourcesListData={ subscribedAPIResourcesListData }
                        closeWizard={ (): void => setIsAuthorizeAPIResourceWizardOpen(false) }
                        handleCreateAPIResource= { handleCreateAPIResource } />
                )
            }
        </Fragment>

    );
};

/**
 * Default props for application roles tab component.
 */
APIAuthorization.defaultProps = {
    "data-componentid": "api-authorization-tab"
};
