/**
 * Copyright (c) 2023, WSO2 LLC. (https://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 LLC. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content.
 */

import { IdentifiableComponentInterface, SBACInterface } from "@wso2is/core/models";
import { Field, FormValue, Forms, useTrigger } from "@wso2is/forms";
import { ContentLoader, Hint, Message, PrimaryButton } from "@wso2is/react-components";
import { ExtendedFeatureConfigInterface } from "apps/console/src/extensions/configs";
import React, { FunctionComponent, ReactElement, useEffect, useState } from "react";
import { Trans, useTranslation } from "react-i18next";
import { Grid } from "semantic-ui-react";
import { Policy } from "../../constants";

/**
 * Prop types for the policy section component.
 */
interface PolicySectionProps extends
    SBACInterface<ExtendedFeatureConfigInterface>, IdentifiableComponentInterface {
    /**
     * Application ID.
     */
    policies: string[];
    /**
     * Is policies loading.
     */
    isPoliciesLoading: boolean;
    /**
     * Is client credentials enabled.
     */
    isClientCredentialsEnabled: boolean;
    /**
     * Is scopes available for update.
     */
    isScopesAvailableForUpdate: boolean;
    /**
     * Set policy section loading.
     */
    setPolicySectionLoading: (isLoading: boolean) => void;
    /**
     * Function to update the policies.
     */
    handleUpdatePolicies: (addedPolicies: string[], removedPolicies: string[]) => void;
}

/**
 * Policy section interface component.
 * 
 * TODO: Add a message notification if no roles have been created while the user have selected role based access policy.
 *
 * @param props - Props injected to the component.
 * @returns Functional component.
 */
export const PolicySection: FunctionComponent<PolicySectionProps> = (
    props: PolicySectionProps
): ReactElement => {

    const {
        policies,
        isPoliciesLoading,
        isClientCredentialsEnabled,
        isScopesAvailableForUpdate,
        setPolicySectionLoading,
        handleUpdatePolicies,
        ["data-componentid"]: componentId
    } = props;

    const { t } = useTranslation();
    const [ submitForm, setSubmitForm ] = useTrigger();
    
    const [ isUserBasedPolicyEnabled, setUserBasedPolicyEnabled ] = useState<boolean>(false);
    const [ isApplicationBasedPolicyEnabled, setApplicationBasedPolicyEnabled ] = useState<boolean>(false);
    const [ defaultUserBasedPolicy, setDefaultUserBasedPolicy ] = useState<Policy>(null);
    const [ enabledPolicies, setEnabledPolicies ] = useState<Policy[]>([]);
    const [ updateButtonDisabled, setUpdateButtonDisabled ] = useState<boolean>(true);

    /**
     * Initialize enabled policies.
     */
    useEffect(() => {
        if (policies) {
            const enabledPolicies: Policy[] = [];

            policies.forEach((policy: string) => {
                if (policy === Policy.ROLE || policy === Policy.CONSENT) {
                    setUserBasedPolicyEnabled(true);
                    enabledPolicies.push(policy);
                    setDefaultUserBasedPolicy(policy);
                } else if (policy === Policy.M2M) {
                    setApplicationBasedPolicyEnabled(true);
                    enabledPolicies.push(Policy.M2M);
                }
            });

            setEnabledPolicies(enabledPolicies);
        }
    }, [ policies ]);

    /**
     * This function is used to handle the policy change.
     * 
     * @param isPure - Is pure.
     * @param values - Form values.
     * @returns `void`
     */
    const handlePolicyChange = (isPure: boolean, values: Map<string, FormValue>): void => {
        const currentPolicies: Policy[] = [];

        // This is to check whether the user based policy is enabled or not within the function.
        let userBasedPolicyEnabled: boolean = false;

        values.forEach((value: FormValue, key: string) => {

            switch (key) {
                case Policy.USER:
                    if (value.length === 1 && value.includes(Policy.USER)) {
                        setUserBasedPolicyEnabled(true);
                        userBasedPolicyEnabled = true;
                    } else {
                        setUserBasedPolicyEnabled(false);
                    }

                    break;
                case Policy.M2M:
                    if (value.length === 1 && value.includes(Policy.M2M)) {
                        setApplicationBasedPolicyEnabled(true);
                        currentPolicies.push(Policy.M2M);
                    } else {
                        setApplicationBasedPolicyEnabled(false);
                    }

                    break;
            
                default:
                    if (userBasedPolicyEnabled) {
                        currentPolicies.push(value as Policy);
                    }

                    break;
            }
        });
        setEnabledPolicies(currentPolicies);

        const checkUpdateButtonDisable: boolean = policies.length === currentPolicies.length &&
            policies.every((element: string) => currentPolicies.includes(element as Policy));

        setUpdateButtonDisabled(checkUpdateButtonDisable);
    };

    /**
     * Function to handle the form submit action.
     * 
     * @returns `void`
     */
    const handleFormSubmit = (): void => {
        let addedPolicies: string[] = [];
        let removedPolicies: string[] = [];

        if (policies.length === 0) {
            addedPolicies = enabledPolicies;
        } else if (enabledPolicies.length === 0) {
            removedPolicies = policies;
        } else {
            enabledPolicies.forEach((policy: Policy) => {
                if (!policies.includes(policy)) {
                    addedPolicies.push(policy);
                }
            });
    
            policies.forEach((policy: string) => {
                if (!enabledPolicies.includes(policy as Policy)) {
                    removedPolicies.push(policy);
                }
            });
        }

        setPolicySectionLoading(true);
        handleUpdatePolicies(addedPolicies, removedPolicies);
    };

    return (
        isPoliciesLoading
            ? <ContentLoader inline="centered" active />
            : (
                <Forms
                    data-componentid={ `${componentId}-form` }
                    uncontrolledForm={ false }
                    onChange={ handlePolicyChange }
                    onSubmit={ handleFormSubmit }
                    submitState={ submitForm }
                    de
                >
                    <Grid>
                        { enabledPolicies.length === 0 && (
                            <Grid.Row columns={ 1 } className={ enabledPolicies.length === 0 ? "pb-0" : "" }>
                                <Grid.Column mobile={ 16 } tablet={ 16 } computer={ 12 }>
                                    <Message
                                        type="warning"
                                        header="Warning"
                                        content={ t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                            ".sections.policySection.messages.noPolicy") }
                                    />
                                </Grid.Column>
                            </Grid.Row>) }
                        { !isClientCredentialsEnabled && isApplicationBasedPolicyEnabled && (
                            <Grid.Row columns={ 1 } className={ "pb-0" }>
                                <Grid.Column mobile={ 16 } tablet={ 16 } computer={ 12 }>
                                    <Message
                                        type="warning"
                                        header="Warning"
                                        content={
                                            (<Trans
                                                i18nKey={
                                                    "extensions:develop.applications.edit.sections.apiAuthorization" +
                                                    ".sections.policySection.messages.noClientCredentials"
                                                }
                                            >
                                                Please <b>enable client credential grant</b> for your application before
                                                applying the application-based policy for secure access to your 
                                                subscribed APIs.
                                            </Trans>)
                                        }
                                    />
                                </Grid.Column>
                            </Grid.Row>) }
                        <Grid.Row columns={ 1 }>
                            <Grid.Column mobile={ 16 } tablet={ 16 } computer={ 12 }>
                                <Field
                                    data-componentid={ `${componentId}-user-based` }
                                    name={ Policy.USER }
                                    type="checkbox"
                                    children={ [
                                        {
                                            label: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                                ".sections.policySection.form.fields.userPolicy.label"),
                                            value: Policy.USER
                                        }
                                    ] }
                                    key={ "key" }
                                    tabIndex={ 3 }
                                    disabled={ !isScopesAvailableForUpdate }
                                    value={ isUserBasedPolicyEnabled ? [ Policy.USER ] : [] }
                                />
                            </Grid.Column>
                        </Grid.Row>
                        <Grid.Row columns={ 1 } className="pt-0 ml-4">
                            <Grid.Column mobile={ 16 } tablet={ 16 } computer={ 14 }>
                                { /* TODO : Add a link to the `Role` tab from the hint text */ }
                                <Field
                                    data-testid="user-mgt-add-user-form-passwordOption-radio-button"
                                    type="radio"
                                    name="userBasedPolicy"
                                    default={ defaultUserBasedPolicy }
                                    children={ [
                                        {
                                            label: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                                ".sections.policySection.form.fields.rbac.label"),
                                            value: Policy.ROLE
                                        }
                                    ] }
                                    tabIndex={ 4 }
                                    disabled={ !isUserBasedPolicyEnabled || !isScopesAvailableForUpdate }
                                />
                                <Hint disabled={ !isUserBasedPolicyEnabled || !isScopesAvailableForUpdate }>
                                    { t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                        ".sections.policySection.form.fields.rbac.hint") }
                                </Hint>
                            </Grid.Column>
                        </Grid.Row>
                        <Grid.Row columns={ 1 } className="pt-0 ml-4">
                            <Grid.Column mobile={ 16 } tablet={ 16 } computer={ 14 }>
                                <Field
                                    data-testid="user-mgt-add-user-form-passwordOption-radio-button"
                                    type="radio"
                                    name="userBasedPolicy"
                                    default={ defaultUserBasedPolicy }
                                    children={ [
                                        {
                                            label: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                                ".sections.policySection.form.fields.consent.label"),
                                            value: Policy.CONSENT
                                        }
                                    ] }
                                    tabIndex={ 4 }
                                    disabled={ !isUserBasedPolicyEnabled || !isScopesAvailableForUpdate }
                                />
                                <Hint disabled={ !isUserBasedPolicyEnabled || !isScopesAvailableForUpdate }>
                                    { t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                        ".sections.policySection.form.fields.consent.hint") }
                                </Hint>
                            </Grid.Column>
                        </Grid.Row>
                        <Grid.Row columns={ 1 }>
                            <Grid.Column mobile={ 16 } tablet={ 16 } computer={ 12 }>
                                <Field
                                    data-componentid={ `${componentId}-user-based` }
                                    name={ Policy.M2M }
                                    type="checkbox"
                                    children={ [
                                        {
                                            label: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                                ".sections.policySection.form.fields.appPolicy.label"),
                                            value: Policy.M2M
                                        }
                                    ] }
                                    key={ "key" }
                                    tabIndex={ 5 }
                                    disabled={ !isScopesAvailableForUpdate }
                                    value={ isApplicationBasedPolicyEnabled ? [ Policy.M2M ] : [] }
                                />
                                <Hint disabled={ !isScopesAvailableForUpdate }>
                                    { t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                        ".sections.policySection.form.fields.appPolicy.hint") }
                                </Hint>
                            </Grid.Column>
                        </Grid.Row>
                        {
                            isScopesAvailableForUpdate && (
                                <Grid.Row columns={ 1 }>
                                    <Grid.Column width={ 16 }>
                                        <PrimaryButton
                                            size="small"
                                            tabIndex={ 6 }
                                            onClick={ setSubmitForm }
                                            data-componentid={ `${componentId}-update-policy-btn` }
                                            disabled={ updateButtonDisabled }
                                        >
                                            { t("extensions:develop.applications.edit.sections.apiAuthorization." +
                                                "sections.policySection.buttons.update") }
                                        </PrimaryButton>
                                    </Grid.Column>
                                </Grid.Row>)
                        }
                    </Grid>
                </Forms>
            )
    );
};

/**
 * Default props for the application advanced configurations form component.
 */
PolicySection.defaultProps = {
    "data-componentid": "policy-section"
};
