/**
 * Copyright (c) 2023, WSO2 LLC. (https://www.wso2.com) All Rights Reserved.
 *
 * This software is the property of WSO2 LLC. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content.
 */

/**
 * @fileOverview This file contains a script to prepare for Static deployment.
 * It modifies the .env file, the deployment.config.json file, etc.
 */

const fs = require("fs");
const path = require("path");

// eslint-disable-next-line no-console
const logger = console;

const PATHS = {
    DEPLOYMENT_CONFIG: path.resolve(path.join(__dirname, "..", "..", "src", "public", "deployment.config.json")),
    ENV_LOCAL: path.resolve(path.join(__dirname, "..", "..", ".env.local"))
};

logger.info("\nPreparing the build for Static app deployment.\n");

/**
 * Reads and modifies the .env file to update the server type, enable pre-auth check, and set the app base path.
 * @param {string} err - Error message, if any.
 * @param {string} data - Contents of the .env file.
 */
fs.readFile(PATHS.ENV_LOCAL, "utf8", (err, data) => {
    if (err) {
        logger.error(err);

        return;
    }

    // Uncomment the lines that match the specified variables and values.
    const result = data
        .replace(/^#?\s*(SERVER_TYPE=)(.*)$/gm, "$1static")
        .replace(/^#?\s*(PRE_AUTH_CHECK=)(.*)$/gm, "$1true")
        .replace(/^#?\s*(APP_BASE_PATH=)(.*)$/gm, "$1app");

    // Overwrite the .env file with the modified content
    fs.writeFile(PATHS.ENV_LOCAL, result, "utf8", (err) => {
        if (err) logger.error(err);
    });
});

/**
 * Reads and modifies the deployment.config.json file to set the app base name.
 * @param {string} err - Error message, if any.
 * @param {string} data - Contents of the deployment.config.json file.
 */
fs.readFile(PATHS.DEPLOYMENT_CONFIG, "utf8", (err, data) => {
    if (err) {
        logger.error(err);

        return;
    }

    const config = JSON.parse(data);
    config.appBaseName = "app";

    // Overwrite the deployment.config.json file with the modified content
    fs.writeFile(PATHS.DEPLOYMENT_CONFIG, JSON.stringify(config, null, 4), "utf8", (err) => {
        if (err) logger.error(err);
    });
});
